'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

let render = exports.render = (() => {
  var _ref = _asyncToGenerator(function* (options, filePath, data = {}) {
    return new Promise(function (resolve, reject) {
      try {
        const rendered = renderFile(options, filePath, data);

        resolve(rendered);
      } catch (err) {
        const message = [`HTML could not render "${filePath}"!`, err];

        if (options.debug) message.push(JSON.stringify(data, null, '  '));

        reject(message.join('\n\n'));
      }
    });
  });

  return function render(_x, _x2) {
    return _ref.apply(this, arguments);
  };
})();

exports.filesForComponent = filesForComponent;
exports.filesForVariant = filesForVariant;

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const fs = require('fs');
const path = require('path');
const assert = require('assert');

const resolveFilePath = (options, embeddingFilePath, filePath) => {
  if (path.isAbsolute(filePath)) {
    const { basedir } = options;
    assert(basedir, 'Please provide a "basedir" option for the HTML adapter to resolve absolute includes.');

    return path.join(basedir, filePath);
  } else {
    const basedir = path.dirname(embeddingFilePath);
    return path.resolve(basedir, filePath);
  }
};

const resolveVariable = (data, varPath, varPathComps) => {
  if (!varPathComps) {
    varPathComps = varPath.split('.');
  }

  if (varPathComps.length === 0) {
    return data;
  } else if (varPathComps.length === 1) {
    const value = data[varPathComps[0]];
    return value || `\${${varPath}}`;
  } else {
    const prop = varPathComps.shift();
    if (data[prop] === undefined) data[prop] = {};
    return resolveVariable(data[prop], varPath, varPathComps);
  }
};

const renderString = (str, data) => str.replace(/\$\{(.+?)\}/g, (match, varPath) => resolveVariable(data, varPath));

const renderFile = (options, filePath, data) => {
  const template = fs.readFileSync(filePath, 'utf-8');
  const html = renderString(template, data);

  const rendered = html.replace(/<!--#\s?include file="(.*?)"(.*?)-->/g, (match, includeFile) => {
    const includePath = resolveFilePath(options, filePath, includeFile);

    return renderFile(options, includePath, data);
  });

  return rendered;
};

function filesForComponent(componentName) {
  return [{
    basename: `${componentName}.html`,
    data: `<div class="${componentName}">\n  <!-- TODO: implement -->\n</div>\n`
  }];
}

function filesForVariant(componentName, variantName) {
  return [{
    basename: `${variantName}.html`,
    data: `<!--#include file="../${componentName}.html" -->\n`
  }];
}